// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

/*
int i;
struct istruct xrange;
struct istruct data;
struct istruct intergrate;
struct istruct left;
struct istruct time;
struct istruct tau;
struct istruct loss;
struct istruct lookup;

inter_load_a(&xrange,"./timedata/CEVoccurrent_sane_axes.dat",0,0);
inter_load_a(&time,"./timedata/TIME_for_CE.txt",0,0);
inter_load_a(&tau,"./timedata/tpv/taulaser2nd_sane.csv",0,1);
	inter_save(&tau,"./tausave.dat");
double charge[xrange.len];
double tottime=time.x[time.len-1]-time.x[0];
double dt=tottime/((double)time.len);
inter_rescale(&xrange,1e-3, 1.0);
//for (i=0;i<xrange.len;i++)
//{
//	printf("%e\n",xrange.x[i]);
//}
double d=180e-9;
int x=0;
int loop=0;

inter_copy(&lookup,&xrange);

for (loop=0;loop<10;loop++)
{
printf("Dooing Loop %d\n",loop);
	for (x=0;x<xrange.len;x++)
	{
		double area=0.06/100.0/100.0;
		inter_load_a(&data,"./timedata/CEVoccurrent_sane.dat",x,xrange.len);
		inter_rescale(&data,1.0, 1.0/area);
		inter_save(&data,"./testsave.dat");
		inter_copy(&intergrate,&data);
		inter_copy(&left,&data);
		inter_copy(&loss,&data);

		double tot=0.0;
		for (i=0;i<intergrate.len;i++)
		{
		tot+=data.data[i]*dt;
		intergrate.data[i]=tot;
		}
		inter_save(&intergrate,"./testsaveintergrate.dat");
		double cap=3.8*epsilon0*area/d;
		double capq=(xrange.x[x]*cap)/1.6e-19;	//charge on capasitor
		capq/=area;
		capq/=d;
		double charge=tot/d;
		charge*=1.0/1.6e-19;
		tot=charge-capq;


		//rewrite as total charge left in the device
		double mytau=0.0;
		for (i=0;i<left.len;i++)
		{


		//printf("%e\n",xrange.x[x]);
		left.data[i]=tot;
		if (loop!=0)
		{
		mytau=inter_get_hard(&lookup,tot);
		loss.data[i]=tot*(1.0-exp(-dt/mytau));
		}else
		{
		loss.data[i]=0.0;
		}

		tot-=data.data[i]*dt/d/1.6e-19;
		}
		double sum_lost=inter_sum(&loss);

		lookup.x[x]=charge-capq-sum_lost;
		lookup.data[x]=inter_get_hard(&tau,xrange.x[x]);


		printf("%e %e %e %e %e %e\n",xrange.x[x],charge,capq,charge-capq,inter_get_hard(&tau,xrange.x[x]),sum_lost);
		inter_save(&left,"./left.dat");
		inter_save(&loss,"./loss.dat");

		inter_free(&loss);
		inter_free(&intergrate);
		inter_free(&data);
		inter_free(&left);

	}
inter_save(&lookup,"./lookuptau.dat");
getchar();
}
inter_free(&xrange);
inter_free(&time);
inter_free(&tau);
exit(0);

*/


void process_ce_data(char *input,char *output)
{
int i;
struct istruct data;

double d=180e-9;
double area=0.06/100.0/100.0;
int remove_cap=TRUE;


double cap=3.8*epsilon0*area/d;
double capq=0.0;

int col=25;

double dt=0.0;

int x=0;

FILE *out=fopen(output,"w");

	for (x=0;x<col;x++)
	{
		printf("loading.... %d\n",x);
		inter_load_a(&data,input,x,col);


		double tot=0.0;

		for (i=1;i<data.len-1;i++)
		{
		dt=(data.x[i+1]-data.x[i-1])/2.0;
		tot+=data.data[i]*dt;
		}


		capq=(data.x[0]*cap)/q;	//charge on capasitor
		capq/=area;
		capq/=d;

		tot/=area;
		tot/=d;
		tot/=q;

		fprintf(out,"%e %e\n",data.data[0],tot-capq);
		inter_free(&data);
	}
//getchar();

fclose(out);

}

