// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include "server.h"
#include <math.h>
#include <gsl/gsl_multimin.h>
#include <sys/types.h>
#include <sys/stat.h>

void fit_set_light(double in)
{
edit_file("sim.inp","",2,in);
}

void fit_set_volt_step(double in)
{
edit_file("sim.inp","",10,in);
}

void fit_set_volt_start(double in)
{
edit_file("sim.inp","",6,in);
}

void fit_set_sim_mode(int in)
{
edit_file_int("sim.inp","",78,in);
}

void fit_set_external_volt(double in)
{
edit_file("sim.inp","",70,in);
}

void fit_set_dt(double in)
{
edit_file("sim.inp","",28,in);
}

void fit_set_rep_rate(int in)
{
edit_file_int("sim.inp","",36,in);
}

void fit_set_pulse_len(int in)
{
edit_file_int("sim.inp","",34,in);
}

void fit_set_tpc_stop(double in)
{
edit_file("sim.inp","",80,in);
}

double fit_run_sims(struct fitvars *fitconfig,struct server *myserver)
{
char tempsim[1000];
char tempexp[1000];
char command[1000];
double voltage;
int i;

gen_dos_fd_gaus_fd();

//getchar();
//printf("about to run for dark\n");
mkdir("./sim", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

if (fitconfig->fit_dark_jv==TRUE)
{
	printf("Running JV\n");
	sprintf(tempsim,"./sim/jv");
	mkdir(tempsim, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
	strcat(tempsim,"/");
	fit_set_light(0.0);
	fit_set_volt_step(0.05);				//change step
	fit_set_volt_start(0.05);				//change vstart
	fit_set_sim_mode(0);					//set simulation mode
	sprintf(command,"cp ./blom_device.inp ./sim.inp %s",tempsim);
	system(command);
	server_add_job(myserver,tempsim);
}



if (fitconfig->dolightfit==TRUE)
{
	sprintf(tempsim,"./sim/jvlight");
	mkdir(tempsim, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
	strcat(tempsim,"/");
	fit_set_light(1000.0);					//light on
	fit_set_volt_step(0.01);				//change step
	fit_set_volt_start(0.3);				//change vstart
	fit_set_sim_mode(0);					//set simulation mode

	sprintf(command,"cp ./blom_device.inp ./sim.inp %s",tempsim);
	system(command);
	server_add_job(myserver,tempsim);
	fit_set_light(0.0);					//light off
}


for (i=0;i<fitconfig->tpcmax;i++)
{

	double suns=0.0;
	double tpc_stop=0.0;
	printf("Running time tpc %d/%d\n",i,fitconfig->tpcmax);
	sprintf(tempsim,"./sim/tpc%d",i);
	sprintf(tempexp,"./exp/tpc%d",i);
	mkdir(tempsim, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

	strcat(tempexp,"/");
	strcat(tempsim,"/");

	FILE *info=fopena(tempexp,"info.inp","r");
	fscanf(info,"%le",&suns);
	fscanf(info,"%le",&voltage);
	fscanf(info,"%le",&tpc_stop);
	fclose(info);

	printf("Doing tpc %d %lf\n",i,voltage);

	fit_set_light(suns);					//light bias
	fit_set_sim_mode(4);					//set simulation mode
	fit_set_external_volt(voltage);				//set externalv

	fit_set_rep_rate(150);					//set rep rate
	fit_set_dt(2.000000e-7);				//set dt
	fit_set_tpc_stop(tpc_stop);				//set tpc timemax
	fit_set_pulse_len(2);					//pulse length
	sprintf(command,"cp ./blom_device.inp ./sim.inp %s",tempsim);
	system(command);
	server_add_job(myserver,tempsim);
}

int ce_dark_max=0;
if (fitconfig->dotimece==TRUE)
{
	struct istruct exp;
	inter_load_a(&exp,"./exp/cedata.dat",0,1);
	double start=exp.x[0];
	double stop=exp.x[exp.len-1];
	double steps=3.0;
	double step=(stop-start)/steps;
	//printf("%e %e\n",start,stop);
	//getchar();
	inter_free(&exp);
	double V=start;

	do
	{
	printf("Running time ce %d\n",ce_dark_max);
	sprintf(tempsim,"./sim/ce%d",ce_dark_max);
	sprintf(tempexp,"./exp/ce%d",ce_dark_max);
	mkdir(tempsim, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

	strcat(tempexp,"/");
	strcat(tempsim,"/");


	fit_set_light(0.0);					//light off
	fit_set_volt_start(0.05);				//change vstart
	fit_set_sim_mode(2);					//set simulation mode
	fit_set_dt(2.4000e-06);					//set dt
	fit_set_rep_rate(25);					//set rep rate
	fit_set_pulse_len(4);					//pulse length
	fit_set_external_volt(V);				//set external voltage
	sprintf(command,"cp ./blom_device.inp ./sim.inp %s",tempsim);
	system(command);

	server_add_job(myserver,tempsim);

	V+=step;
	ce_dark_max++;
	}while(ce_dark_max<=steps);


	//run_simulation(tempsim);
}


int ce_light_max=0;
if (fitconfig->dotimecelight==TRUE)
{
	struct istruct exp;
	inter_load_a(&exp,"./exp/celight.dat",0,1);
	double start=exp.x[0];
	double stop=exp.x[exp.len-1];
	double steps=3.0;
	double step=(stop-start)/steps;
	//printf("%e %e\n",start,stop);
	//getchar();
	inter_free(&exp);
	double V=start;

	do
	{
	printf("Running time ce light %d\n",ce_light_max);
	sprintf(tempsim,"./sim/celight%d",ce_light_max);
	sprintf(tempexp,"./exp/celight%d",ce_light_max);
	mkdir(tempsim, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

	strcat(tempexp,"/");
	strcat(tempsim,"/");


	fit_set_light(1000.0);					//One sun
	fit_set_volt_start(0.05);		//change vstart
	fit_set_sim_mode(2);					//set simulation mode
	fit_set_dt(2.4000e-06);					//set dt
	fit_set_rep_rate(25);					//set rep rate
	fit_set_pulse_len(4);					//pulse length
	fit_set_external_volt(V);				//set external voltage
	sprintf(command,"cp ./blom_device.inp ./sim.inp %s",tempsim);
	system(command);

	server_add_job(myserver,tempsim);

	V+=step;
	ce_light_max++;
	}while(ce_light_max<=steps);


	//run_simulation(tempsim);
}
//fitconfig->tpclightmax

print_jobs(myserver);
server_run_jobs(myserver);

//printf("here\n");
//getchar();
if (fitconfig->dotimece==TRUE)
{
	mkdir("./sim/ce", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
	FILE *r;
	r=fopen("./sim/ce/realce.dat","w");
	fclose(r);
	double nread=0.0;
	double V=0.0;
	for (i=0;i<ce_dark_max;i++)
	{
		sprintf(tempsim,"./sim/ce%d/cej.dat",i);
		r=fopen(tempsim,"r");
		fscanf(r,"%le %le",&V,&nread);
		fclose(r);
		r=fopen("./sim/ce/realce.dat","a");
		fprintf(r,"%le %le\n",V,nread);
		fclose(r);
	}
}

if (fitconfig->dotimecelight==TRUE)
{
	mkdir("./sim/celight", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
	FILE *r;
	r=fopen("./sim/celight/realce.dat","w");
	fclose(r);
	double nread=0.0;
	double V=0.0;
	for (i=0;i<ce_light_max;i++)
	{
		sprintf(tempsim,"./sim/celight%d/cej.dat",i);
		r=fopen(tempsim,"r");
		fscanf(r,"%le %le",&V,&nread);
		fclose(r);
		r=fopen("./sim/celight/realce.dat","a");
		fprintf(r,"%le %le\n",V,nread);
		fclose(r);
	}
}

return get_all_error(fitconfig);
}


