// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include "hist.h"
#include <math.h>
#include <openssl/md5.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>

#ifndef mindump
#define test_dist_n
#define test_dist_p


double pick[1000];
double pick_x[1000];
double pick_start=-7;
double pick_stop=0;
double pick_dx;
#endif

double Eg=1.1;

static int gendos=TRUE;
static struct dosconfig confige;
static struct dosconfig configh;

double draw_Ec=-3.8;
double draw_Ev=-1.0400-3.8;

void dos_double_res()
{
/*double number;
FILE *in=fopen("./srhbandp.inp","r");
FILE *out=fopen("./srhbandp.new","w");
do
{
fscanf(in,"%le",&number);
fprintf(out,"%le\n",number);
fprintf(out,"%le\n",number);
}while(!feof(in));
fclose(in);
fclose(out);
*/
int i;
for (i=1;i<=80;i++)
{
printf("p srhbandp.inp %d 1\n",i);
}
}

void md5write(char *file_name)
{

FILE *file;
char *buffer;
unsigned long len;
char chkfile[100];
sprintf(chkfile,"md5.%s.dat",file_name);
file = fopen(file_name, "rb");

fseek(file, 0, SEEK_END);
len=ftell(file);
fseek(file, 0, SEEK_SET);


buffer=(char *)malloc(len+1);



fread(buffer, len, 1, file);
fclose(file);

unsigned char md5[16];
MD5((const unsigned char*)buffer, len,md5);
char temp[20];
sprintf(temp,"%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x",md5[0],md5[1],md5[2],md5[3],md5[4],md5[5],md5[6],md5[7],md5[8],md5[9],md5[10],md5[11],md5[12],md5[13],md5[14],md5[15]);

printf("%s\n",temp);
free(buffer);

file=fopen(chkfile,"w");
fprintf(file,"%s\n",temp);
fclose(file);
}

int md5check(char *file_name)
{

FILE *file;
char *buffer;
unsigned long len;

file=fopen("./dosn_gau.dat","r");
if (!file)
{
return FALSE;
}
fclose(file);

file=fopen("./dosp_gau.dat","r");
if (!file)
{
return FALSE;
}
fclose(file);

char chkfile[100];
sprintf(chkfile,"md5.%s.dat",file_name);

file = fopen(file_name, "rb");

fseek(file, 0, SEEK_END);
len=ftell(file);
fseek(file, 0, SEEK_SET);


buffer=(char *)malloc(len+1);



fread(buffer, len, 1, file);
fclose(file);

unsigned char md5[16];

MD5((const unsigned char*)buffer, len,md5);

char newcheck[40];
strcpy(newcheck,"hello");
char fromfile[40];
//printf("%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x\n",md5[0],md5[1],md5[2],md5[3],md5[4],md5[5],md5[6],md5[7],md5[8],md5[9],md5[10],md5[11],md5[12],md5[13],md5[14],md5[15]);
sprintf(newcheck,"%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x%x",md5[0],md5[1],md5[2],md5[3],md5[4],md5[5],md5[6],md5[7],md5[8],md5[9],md5[10],md5[11],md5[12],md5[13],md5[14],md5[15]);

//printf("%s\n",temp);

file=fopen(chkfile,"r");
if (!file)
{
return FALSE;
}

fscanf(file,"%s\n",fromfile);
fclose(file);

if (strcmp(newcheck,fromfile)==0)
{
return TRUE;
}else
{
return FALSE;
}
free(buffer);
return 0;
}

void dump_qe()
{
struct istruct n;
struct istruct p;
int fit_len=0;

fit_len=get_file_len("./dosoutn.dat")-1;
inter_load(&n,"./dosoutn.dat",fit_len,0,1);
fit_len=get_file_len("./dosoutp.dat")-1;
inter_load(&p,"./dosoutp.dat",fit_len,0,1);
inter_swap(&p);


double start=n.x[0];
double stop=n.x[n.len-1];
double step=5e-3;
double pos=0.0;
double hf=0.0;
double hf_step=1e-2;
double re=0.0;
double rh=0.0;
double rhored=0.0;
double rhored_tot=0.0;
double Ec=-3.8;
double Ef=Ec-0.5;
pos=start;
double max=0.0;
FILE *qe=fopen("qe.dat","w");
hf_step=1e-1;
do
{
pos=start;
rhored_tot=0.0;
	do
	{
	re=inter_get(&n,pos+hf);///(exp(q*(pos-Ef)/(kb*300.0))+1);
	rh=inter_get(&p,pos);//*(1-1.0/(exp(q*(pos-Ef)/(kb*300.0))+1));
	rhored=re*rh;
	//printf("%e %e %e %e\n",pos,rhored,re,rh);
	rhored_tot+=rhored;
	pos+=step;
	}while(pos<stop);

if (rhored_tot>max) max=rhored_tot;
//getchar();
hf+=hf_step;
printf("%e\n",hf);
}while(hf<2.0);

hf=0;
hf_step=1e-2;

do
{
pos=start;
rhored_tot=0.0;
	do
	{
	re=inter_get(&n,pos+hf);///(exp(q*(pos-Ef)/(kb*300.0))+1);
	rh=inter_get(&p,pos);//*(1-1.0/(exp(q*(pos-Ef)/(kb*300.0))+1));
	rhored=re*rh;
	rhored_tot+=rhored;

	pos+=step;
	}while(pos<stop);
//getchar();
fprintf(qe,"%e %e\n",hf,rhored_tot/max);
hf+=hf_step;
printf("%e\n",hf);
}while(hf<2.0);

fclose(qe);
inter_free(&n);
inter_free(&p);
}

void pick_init()
{
#ifndef mindump
int i;
pick_dx=(pick_stop-pick_start)/1000.0;
double pos=pick_start;
for (i=0;i<1000;i++)
{
pick[i]=0.0;
pick_x[i]=pos;
pos+=pick_dx;
}
#endif
}

void pick_add(double x,double value)
{
#ifndef mindump
int pos;
for (pos=0;pos<1000;pos++)
{
if (pick_x[pos]>x) break;
}
if (pos<0.0) return;
if (pos>999) return;
if (value>pick[pos]) pick[pos]=value;
#endif
}

void pick_dump()
{
#ifndef mindump
int i;
FILE *out;
out=fopen("./gaus.dat","w");
double pos=pick_start;
for (i=0;i<1000;i++)
{
fprintf(out,"%e %e\n",pick_x[i],pick[i]);
pos+=pick_dx;
}
fclose(out);
#endif
}

void gen_do(struct dosconfig *in,struct dosconfig *in2,char * outfile,int electrons)
{
#ifdef dos_test_stats
FILE *freetest;
if (electrons==TRUE)
{
	freetest=fopen("./freetestn.dat","w");
	fclose(freetest);
}else
{
	freetest=fopen("./freetestp.dat","w");
	fclose(freetest);
}
#endif

FILE *out;
double tstart=0.0;
double tstop=0.0;
double tsteps=0.0;
double dt=0.0;
double xpos=0.0;
double tpos=0.0;
int t=0;
int x=0;
int band;
double *srh_r1=(double *)malloc(sizeof(double)*in->srh_bands);
double *srh_r2=(double *)malloc(sizeof(double)*in->srh_bands);
double *srh_r3=(double *)malloc(sizeof(double)*in->srh_bands);
double *srh_r4=(double *)malloc(sizeof(double)*in->srh_bands);
double *srh_n=(double *)malloc(sizeof(double)*in->srh_bands);
double *srh_den=(double *)malloc(sizeof(double)*in->srh_bands);
double *srh_dE_sum=(double *)malloc(sizeof(double)*in->srh_bands);
double *srh_read=(double *)malloc(sizeof(double)*in->srh_bands);

double *srh_x=(double *)malloc(sizeof(double)*in->srh_bands);
double *srh_mid=(double *)malloc(sizeof(double)*in->srh_bands);

if (in->dostype==dos_read)
{
FILE *dosread;
	if (electrons==TRUE)
	{
		dosread=fopen("./srhbandn.inp","r");
	}else
	{
		dosread=fopen("./srhbandp.inp","r");
	}

	if (dosread==NULL)
	{
		printf("can not open srhband file\n");
		exit(0);
	}

	for (band=0;band<in->srh_bands;band++)
	{
		fscanf(dosread,"%le",&(srh_read[band]));
		printf("%le\n",srh_read[band]);
		srh_read[band]=fabs(srh_read[band]);
	}


	fclose(dosread);
}
int e=0;
double Esteps=4000;
double Estart=-1.0*q;
double Estop=0.0*q;
double dE=(Estop-Estart)/Esteps;
double E=Estart;
double rho=0.0;
double rho2=0.0;
double sum=0.0;
double sum_w=0.0;
double f=0.0;
double last_n0=0;
double *xmesh=NULL;

out=fopen(outfile,"w");
tstart=in->Tstart;
tstop=in->Tstop;
tsteps=in->Tsteps;
dt=(tstop-tstart)/tsteps;
//printf("%le %le\n",in->nstop,in->nstart);
//getchar();
double dxr=(in->nstop-in->nstart)/((double)in->npoints);

xpos=in->nstart;
tpos=tstart;
t=0;
x=0;
fprintf(out,"%d %d %d %lf %le %le %le %le %le %le\n",(int)in->npoints,(int)tsteps,in->srh_bands,in->sigma,in->mu,in->srh_vth,in->srh_sigman,in->srh_sigmap,in->Nc,in->Nv);

double srh_pos=in->srh_start;
double srh_delta=fabs(in->srh_start)/(double)(in->srh_bands);
xmesh=(double *)malloc(sizeof(double)*in->npoints);

for (x=0;x<in->npoints;x++)
{
	fprintf(out,"%e\n",xpos);
	xmesh[x]=xpos;
	xpos+=dxr;
}

for (t=0;t<tsteps;t++)
{
	fprintf(out,"%e\n",tpos);
	tpos+=dt;
}

for (band=0;band<in->srh_bands;band++)
{
	srh_pos+=srh_delta/2.0;
	srh_mid[band]=srh_pos*q;
	fprintf(out,"%e\n",srh_pos);
	srh_pos+=srh_delta/2.0;
	srh_x[band]=srh_pos*q;
}

tpos=tstart;

#ifdef test_dist_n
FILE *munfile=fopen("./munfile.dat","w");
if (munfile==NULL)
{
	printf("problem\n");
	exit(0);
}

tpos=300;



FILE *plotbands;
plotbands=fopen("./plotbandsn.dat","w");

FILE *plotbandsfree;
plotbandsfree=fopen("./plotbandsfreen.dat","w");




FILE *rod=fopen("./gau_test_n.dat","w");

#endif


double sigma=in->sigma*q;

#ifdef test_dist_n
int first=FALSE;
FILE *dosout;
if (electrons==TRUE)
{
dosout=fopen("./dosoutn.dat","w");
}else
{
dosout=fopen("./dosoutp.dat","w");
}
#endif

int srh_band=0;
double srh_E=srh_mid[0];
double srh_f=0.0;
double f2=0.0;
double sum2=0.0;

for (t=0;t<tsteps;t++)
{
printf("%d/%d\n",t,(int)tsteps);

	for (x=0;x<in->npoints;x++)
	{
		xpos=xmesh[x];
		E=Estart;
		sum=0.0;
		sum2=0.0;
		sum_w=0.0;

		for (srh_band=0;srh_band<(in->srh_bands);srh_band++)
		{
			srh_r1[srh_band]=0.0;
			srh_r2[srh_band]=0.0;
			srh_r3[srh_band]=0.0;
			srh_r4[srh_band]=0.0;
			srh_n[srh_band]=0.0;
			srh_den[srh_band]=0.0;
			srh_dE_sum[srh_band]=0.0;
		}

		srh_band=0;
		//printf("start\n");
		for (e=0;e<(Esteps);e++)
		{
			//printf("%le %le %le %le %le %le\n",E/q,srh_x[srh_band]/q,srh_n[srh_band],rho,f,dE);
			//printf("%d %le\n",srh_band,E/q);
			if (E>srh_x[srh_band])
			{

				if (srh_band<((in->srh_bands)-1))
				{
				//printf("%d %le \n",srh_band,E/q);
				srh_band++;
				//getchar();
				}

			}

			srh_E=srh_mid[srh_band];

			f=1.0/(1.0+exp((E/q-xpos)*q/(kb*tpos)));
			f2=1.0/(1.0+exp((E/q-(-1.0*(Eg+xpos)))*q/(kb*tpos)));
			//printf("%le %le %le %le\n",f2,-1.0*(Eg+xpos),Eg,xpos);
			//getchar();
			srh_f=1.0/(1.0+exp((srh_E/q-xpos)*q/(kb*tpos)));
			//printf("%le %le\n",xpos,E/q);
			if (in->dostype==dos_gau)
			{
				rho=in->Ngaus*(1.0/(sqrt(2.0*pi)*sigma))*exp(-pow(((E)/(sqrt(2.0)*sigma)),2.0));
			}else
			if (in->dostype==dos_fd)
			{
				if (E>0)
				{
					rho=(sqrt(E)/(4.0*pi*pi))*pow((2.0*in->m*m0)/(hbar*hbar),3.0/2.0);
				}else
				{
					rho=0.0;
				}

			}else
			if (in->dostype==dos_exp_fd)
			{

				if (E>0)
				{
					rho=2.0*(sqrt(E)/(4.0*pi*pi))*pow((2.0*in->m*m0)/(hbar*hbar),3.0/2.0);
					rho2=2.0*(sqrt(E)/(4.0*pi*pi))*pow((2.0*in2->m*m0)/(hbar*hbar),3.0/2.0);
				}else
				{
					rho=in->Nt*exp((E)/(in->Et*q));
					rho2=in2->Nt*exp((E)/(in2->Et*q));
				}
				//if (rho>1e60)	rho=0.0;
				//if (rho2>1e60)	rho2=0.0;

			}else
			if ((in->dostype==dos_exp)||(in->dostype==dos_mb))
			{

				rho=in->Nt*exp((E)/(in->Et*q));
				rho2=in2->Nt*exp((E)/(in2->Et*q));

				//if (rho>1e40)	rho=0.0;
				//if (rho2>1e40)	rho2=0.0;

			}else
			if (in->dostype==dos_read)
			{
				rho=srh_read[srh_band];
				//printf("%le %le\n",E/q,in->srh_cut);
				if (E<q*in->srh_cut)
				{
					rho=1e20;
				}
			}



			if (x==0)
			{
				pick_add(E/q-3.8,rho);
				#ifdef test_dist_n
				if (electrons==TRUE)
				{
				fprintf(dosout,"%le %le\n",E/q-3.8,rho);
				}else
				{
				fprintf(dosout,"%le %le\n",-E/q-3.8-Eg,rho);
				}
				#endif
			}

			if (E>0)
			{
				sum+=rho*f*dE;
				sum2+=rho2*f2*dE;
				//printf("%le %le\n",rho2,f2);
			}else
			{
				//printf("%le %le\n",rho,f);
				//rho=1e47;
				if (electrons==TRUE)
				{
					srh_r1[srh_band]+=in->srh_vth*in->srh_sigman*rho*(1.0-srh_f)*dE;
					srh_r2[srh_band]+=in->srh_vth*in->srh_sigman*in->Nc*exp(srh_E/(300.0*kb))*rho*srh_f*dE;//in->srh_vth*srh_sigman*rho*(1.0-srh_f)*dE;//
					srh_r3[srh_band]+=in->srh_vth*in->srh_sigmap*rho*srh_f*dE;
					srh_r4[srh_band]+=in->srh_vth*in->srh_sigmap*in->Nv*exp((-1.0*q-srh_E)/(300.0*kb))*rho*(1.0-srh_f)*dE;//in->srh_vth*srh_sigmap*rho*srh_f*dE;//
					srh_n[srh_band]+=rho*srh_f*dE;
					srh_den[srh_band]+=rho*dE;
					srh_dE_sum[srh_band]+=dE;
				}else
				{
					srh_r1[srh_band]+=in->srh_vth*in->srh_sigmap*rho*(1.0-srh_f)*dE;
					srh_r2[srh_band]+=in->srh_vth*in->srh_sigmap*in->Nv*exp(srh_E/(300.0*kb))*rho*srh_f*dE;//in->srh_vth*srh_sigman*rho*(1.0-srh_f)*dE;//
					srh_r3[srh_band]+=in->srh_vth*in->srh_sigman*rho*srh_f*dE;
					srh_r4[srh_band]+=in->srh_vth*in->srh_sigman*in->Nc*exp((-1.0*q-srh_E)/(300.0*kb))*rho*(1.0-srh_f)*dE;//in->srh_vth*srh_sigmap*rho*srh_f*dE;//
					srh_n[srh_band]+=rho*srh_f*dE;
					srh_den[srh_band]+=rho*dE;
					srh_dE_sum[srh_band]+=dE;
				}
			}

			//printf("%le\n",E);

			#ifdef test_dist_n
			if (E>=0.0)
			{


				if (t==0)
				if ((x%10)==0)
				if ((e%2)==0)
				{
					fprintf(plotbandsfree,"%.20le %.20le\n",E,rho*f*dE);
					if (first==FALSE) fprintf(plotbands,"%.20le %.20le\n",E,rho*f*dE);
					first=TRUE;
				}

			}else
			{

				if (t==0)
				if ((x%10)==0)
				if ((e%10)==0)
				{
					fprintf(plotbands,"%.20le %.20le\n",E,rho*f*dE);
				}

			}
			#endif

			//printf("%d %e %e %e %e %e\n",e,E/q,rho,f,gau,sum);

			E+=dE;
		}
		//printf("stop\n");
		//getchar();
		#ifdef test_dist_n
		fprintf(plotbands,"\n");
		fprintf(plotbandsfree,"\n");
		#endif

		if (x==0)
		{
			for (band=0;band<in->srh_bands;band++)
			{
				fprintf(out,"%le\n",srh_den[band]);
			}

			#ifdef dump_bs
			FILE *bandsdump;
			if (electrons==TRUE)
			{
				bandsdump=fopen("./srhbandn.inp","w");
				for (band=0;band<in->srh_bands;band++)
				{
					fprintf(bandsdump,"%le\n",srh_den[band]/srh_dE_sum[band]);
				}
				fclose(bandsdump);
			}else
			{
				bandsdump=fopen("./srhbandp.inp","w");
				for (band=0;band<in->srh_bands;band++)
				{
					fprintf(bandsdump,"%le\n",srh_den[band]/srh_dE_sum[band]);
				}
				fclose(bandsdump);
			}
			#endif
		}

		if ((in->dostype==dos_mb)||(in->dostype==dos_read))
		{
			if (electrons==TRUE)
			{
				sum=in->Nc*exp((xpos*q)/(kb*tpos));
				sum2=in->Nv*exp((-(Eg+xpos)*q)/(kb*tpos));
			}else
			{
				sum=in->Nv*exp((xpos*q)/(kb*tpos));
				sum2=in->Nc*exp((-(Eg+xpos)*q)/(kb*tpos));
			}

		}

		double w0=sum/((sum-last_n0)/(dxr));
		if (x==0) w0=kb*300.0/q;
		fprintf(out,"%.20le %.20le ",sum,w0);
		last_n0=sum;


		#ifdef dos_test_stats
			if (electrons==TRUE)
			{
				freetest=fopen("./freetestn.dat","a");
				//fprintf(freetest,"%.20le %.20le %.20le %.20le %.20le\n",xpos,sum,in->Nc*exp((xpos*q)/(kb*tpos)),sum2,in->Nv*exp((-(Eg+xpos)*q)/(kb*tpos)));
				fprintf(freetest,"%.20le %.20le %.20le\n",xpos,(3.0/2.0)*w0,(3.0/2.0)*kb*300.0/q);
				fclose(freetest);
			}else
			{
				freetest=fopen("./freetestp.dat","a");
				//fprintf(freetest,"%.20le %.20le %.20le %.20le %.20le\n",xpos,sum,in->Nv*exp((xpos*q)/(kb*tpos)),sum2,in->Nc*exp((-(Eg+xpos)*q)/(kb*tpos)));
				fprintf(freetest,"%.20le %.20le %.20le\n",xpos,(3.0/2.0)*w0,(3.0/2.0)*kb*300.0/q);
				fclose(freetest);
			}
		#endif

		for (srh_band=0;srh_band<in->srh_bands;srh_band++)
		{
			fprintf(out,"%.20le %.20le %.20le %.20le %.20le ",srh_r1[srh_band],srh_r2[srh_band],srh_r3[srh_band],srh_r4[srh_band],srh_n[srh_band]);
		}

		fprintf(out,"\n");


		#ifdef test_dist_n
		fprintf(rod,"\n");
		//if (t==0) fprintf(munfile,"%le %le\n",n_tot,mu_tot/n_tot);
		#endif


	}

//getchar();

tpos+=dt;
}
#ifdef test_dist_n
	fclose(rod);
	fclose(plotbands);
	fclose(plotbandsfree);
	fclose(dosout);
	fclose(munfile);
#endif


fclose(out);
free(xmesh);
free(srh_r1);
free(srh_r2);
free(srh_r3);
free(srh_r4);
free(srh_n);
free(srh_den);
free(srh_dE_sum);
free(srh_read);

free(srh_x);
free(srh_mid);
return;
}


void gen_dos_fd_gaus_n()
{
printf("Electrons....\n");
gen_do(&confige,&configh,"./dosn_gau.dat",TRUE);
}

void gen_dos_fd_gaus_p()
{
printf("Holes....\n");
gen_do(&configh,&confige,"./dosp_gau.dat",FALSE);
}



void gen_load_dos()
{
FILE *in;
char name[100];
in=fopen ("./dos.inp","r");
if (in==NULL)
{
printf("Density of states config not found\n");
exit(0);
}
fscanf(in,"%s",name);
fscanf(in,"%d",&(confige.dostype));
configh.dostype=confige.dostype;
fscanf(in,"%s",name);
fscanf(in,"%lf",&(confige.m));
fscanf(in,"%s",name);
fscanf(in,"%lf",&(configh.m));


fscanf(in,"%s",name);
fscanf(in,"%lf",&(confige.Nt));
fscanf(in,"%s",name);
fscanf(in,"%lf",&(configh.Nt));
confige.Nt=fabs(confige.Nt);
configh.Nt=fabs(configh.Nt);
fscanf(in,"%s",name);
fscanf(in,"%lf",&(confige.Et));
fscanf(in,"%s",name);
fscanf(in,"%lf",&(configh.Et));

confige.Et=fabs(confige.Et);
configh.Et=fabs(configh.Et);

if (confige.Et<2e-3) confige.Et=2e-3;
if (configh.Et<2e-3) configh.Et=2e-3;

fscanf(in,"%s",name);
fscanf(in,"%d",&gendos);
//printf("%d\n",gendos);

fscanf(in,"%s",name);
fscanf(in,"%le",&(confige.mu));
fscanf(in,"%s",name);
fscanf(in,"%le",&(configh.mu));
confige.mu=fabs(confige.mu);
configh.mu=fabs(configh.mu);


fscanf(in,"%s",name);
fscanf(in,"%le",&confige.sigma);
confige.sigma=fabs(confige.sigma);
fscanf(in,"%s",name);
fscanf(in,"%le",&configh.sigma);
configh.sigma=fabs(configh.sigma);

fscanf(in,"%s",name);
fscanf(in,"%le",&confige.Ngaus);
confige.Ngaus=fabs(confige.Ngaus);
fscanf(in,"%s",name);
fscanf(in,"%le",&configh.Ngaus);
configh.Ngaus=fabs(configh.Ngaus);
fscanf(in,"%s",name);
fscanf(in,"%le",&confige.Tstart);
fscanf(in,"%s",name);
fscanf(in,"%le",&confige.Tstop);
fscanf(in,"%s",name);
fscanf(in,"%d",&confige.Tsteps);

configh.Tstart=confige.Tstart;
configh.Tstop=confige.Tstop;
configh.Tsteps=confige.Tsteps;

fscanf(in,"%s",name);
fscanf(in,"%le",&(confige.nstart));
fscanf(in,"%s",name);
fscanf(in,"%le",&(confige.nstop));
fscanf(in,"%s",name);
fscanf(in,"%d",&(confige.npoints));

fscanf(in,"%s",name);
fscanf(in,"%le",&(configh.nstart));
fscanf(in,"%s",name);
fscanf(in,"%le",&(configh.nstop));
fscanf(in,"%s",name);
fscanf(in,"%d",&(configh.npoints));

fscanf(in,"%s",name);
fscanf(in,"%d",&(confige.srh_bands));
configh.srh_bands=confige.srh_bands;

fscanf(in,"%s",name);
fscanf(in,"%le",&(confige.srh_start));
configh.srh_start=confige.srh_start;

fscanf(in,"%s",name);
fscanf(in,"%le",&(confige.srh_sigman));
confige.srh_sigman=fabs(confige.srh_sigman);

fscanf(in,"%s",name);
fscanf(in,"%le",&(confige.srh_sigmap));
confige.srh_sigmap=fabs(confige.srh_sigmap);

fscanf(in,"%s",name);
fscanf(in,"%le",&(confige.srh_vth));
confige.srh_vth=fabs(confige.srh_vth);
if (confige.srh_vth<1e2) confige.srh_vth=1e2;

fscanf(in,"%s",name);
fscanf(in,"%le",&(configh.srh_sigman));
configh.srh_sigman=fabs(configh.srh_sigman);

fscanf(in,"%s",name);
fscanf(in,"%le",&(configh.srh_sigmap));
configh.srh_sigmap=fabs(configh.srh_sigmap);

fscanf(in,"%s",name);
fscanf(in,"%le",&(configh.srh_vth));

configh.srh_vth=fabs(configh.srh_vth);
if (configh.srh_vth<1e2) configh.srh_vth=1e2;

fscanf(in,"%s",name);
fscanf(in,"%le",&(confige.Nc));

fscanf(in,"%s",name);
fscanf(in,"%le",&(confige.Nv));

fscanf(in,"%s",name);
fscanf(in,"%le",&(confige.srh_cut));
confige.srh_cut=-fabs(confige.srh_cut);
configh.srh_cut=confige.srh_cut;

confige.Nc=fabs(confige.Nc);
confige.Nv=fabs(confige.Nv);
if (confige.Nc<1e24) confige.Nc=1e24;
if (confige.Nv<1e24) confige.Nv=1e24;
confige.m=pow(confige.Nc/2.0,2.0/3.0)*hp*hp/kb/300.0/m0/2.0/pi;
configh.m=pow(confige.Nv/2.0,2.0/3.0)*hp*hp/kb/300.0/m0/2.0/pi;
//printf("%le %le\n",confige.m,configh.m);
//getchar();
//(sqrt(E)/(4.0*pi*pi))*pow((2.0*in->m*m0)/(hbar*hbar),3.0/2.0)

configh.Nc=confige.Nc;
configh.Nv=confige.Nv;
//printf("%le\n",configh.Nv);
//getchar();
fclose(in);
}

void gen_dos_fd_gaus_fd()
{
pick_init();

gen_load_dos();
int gennewdos=FALSE;
if (md5check("dos.inp")==FALSE) gennewdos=TRUE;
if (confige.dostype==dos_read)
{
	if (md5check("srhbandn.inp")==FALSE) gennewdos=TRUE;
	if (md5check("srhbandp.inp")==FALSE) gennewdos=TRUE;
}

if ((gendos==TRUE)&&(gennewdos==TRUE))
{
	server_add_job(&globalserver,"gendosn");
	server_add_job(&globalserver,"gendosp");
	print_jobs(&globalserver);
	server_run_jobs(&globalserver);

pick_dump();
md5write("dos.inp");

if (confige.dostype==dos_read)
{
md5write("srhbandn.inp");
md5write("srhbandp.inp");
}

printf("Finished generating....\n");

}
}


