// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include "sim.h"
#include "server.h"

#include <math.h>

#ifdef makegui
#include <gtk/gtk.h>
#include "gui.h"
#endif

struct device cell;

void printdebug(char *text)
{
#ifdef makegui
//printf("'%s'\n",gui.filename);
//getchar();
if (gui.show==TRUE)
{
  GtkTextBuffer *tbuffer;
  GtkTextIter ei;

  tbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(gui.textview));
  gtk_text_buffer_get_end_iter(tbuffer, &ei);
  gtk_text_buffer_insert(tbuffer, &ei, text, -1);
gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (gui.textview),
                                      &ei, 0.0, FALSE, 0, 0);
}else
{
#endif
printf("%s",text);
#ifdef makegui
}
#endif
}


#ifdef makegui
struct gui gui;

struct gui* get_gui()
{
return &gui;
}



void about_box_show(GtkWidget *widget, gpointer user_data)
{

const gchar* authors[] = {
		"Roderick MacKenzie <rod@roderickmackenzie.eu> or <r.c.i.mackenzie@googlemail.com> or <r.mackenzie@imperial.ac.uk>",
		NULL
    };

	GdkPixbuf *logo = gdk_pixbuf_new_from_file ("sun.png", NULL);

	gtk_show_about_dialog(NULL,
	                     "name", "Solarsim",
			     "program-name" ,"Solarsim",
	                     "version", "V 0.1",
	                     "copyright", "Copyright \xc2\xa9 2008 Roderick MacKenzie",
	                     "comments",  "Organic solar cell simulator",
	                     "website", "<www.roderickmackenzie.eu>",
	                     "authors", authors,
	                     "license", "SolarSim V0.1 License:\n This software is not open source and the source\ncode or the executable may not be distributed.   I do not mind you using\n this code for your academic research. However, due to the large amount of\n time I have invested in the software I must insist that I am included as\n an author on any resulting publications.  rod@roderickmackenzie.eu",
	                  	"logo", logo,
	                     NULL);
}








void warn_dialog(gchar *message,char* type)
{
  gchar *markup;
  GtkDialog *dialog;
  GtkHBox *hbox;
  GtkImage *icon;
  GtkLabel *text;
//  GtkWidget *mainwindow;
  markup = g_strdup_printf("%s",
			   message);
	//mainwindow = glade_xml_get_widget (glade_xml, "MainWindow");

  dialog = GTK_DIALOG(gtk_dialog_new_with_buttons(
                      "Solar sim", NULL, GTK_DIALOG_MODAL,
                      "OK", GTK_RESPONSE_OK, NULL));

  gtk_dialog_set_default_response(dialog, GTK_RESPONSE_OK);
  g_signal_connect_swapped(dialog, "response", G_CALLBACK(gtk_widget_destroy),
                           GTK_WIDGET(dialog));

  hbox = (GtkHBox*)g_object_new(GTK_TYPE_HBOX, "border-width", 8, NULL);
  icon = (GtkImage*)g_object_new(GTK_TYPE_IMAGE,
		      "stock", type,
                      "icon-size", GTK_ICON_SIZE_DIALOG,
                      "xalign", 0.5, "yalign", 0.5, NULL);

  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(icon), FALSE, FALSE, 0);
  text = (GtkLabel*)g_object_new(GTK_TYPE_LABEL,
                      "wrap", TRUE,
                      "label", markup,
                      "use-markup", TRUE, NULL);
  g_free(markup);

  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(text), TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(dialog->vbox), GTK_WIDGET(hbox),
                     FALSE, FALSE, 0);
  gtk_widget_show_all(GTK_WIDGET(dialog));
}




double get_error(struct device *in,double *b)
{
double tot=0.0;
int i;
for (i=0;i<in->ymeshpoints;i++)
{

	tot+=mod(b[i]);
}
return tot;
}



void on_window_destroy (GtkObject *object, gpointer user_data)
{
    gtk_main_quit ();
}

void on_simulation_run (GtkObject *object, gpointer user_data)
{
printdebug( "Running simulation\n");
cell.stop=FALSE;
cell.onlypos=FALSE;

    run_simulation("");
warn_dialog("<big>Done!</big>\n\nThe simulation has finished.",GTK_STOCK_DIALOG_INFO);
}


void equ_click (GtkObject *object, gpointer user_data)
{
printdebug( "Calculating equilibrium\n");
cell.stop=FALSE;
cell.onlypos=TRUE;

    run_simulation("");

FILE *plot = popen("gnuplot -persist","w");
fprintf(plot, "set terminal x11 title 'Solarsim - equilibrium band structure'\n");
fprintf(plot, "clear\n");
fprintf(plot, "set data style lines\n");
fprintf(plot, "load 'equ.plot'\n");
fflush(plot);
fclose(plot);
}

void on_show_configure_click (GtkObject *object, gpointer user_data)
{
GtkWidget * widget;
	gtk_widget_show(gui.configure);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "ex1"));

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON( widget), gui_device.enable_ex);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "plotresults"));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "voltageorlight"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON( widget), gui_device.pulsevoltage);

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "plotresults"));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON( widget), gui_device.enable_plot);

	//widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "timedomain"));
	//gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON( widget), gui_device.time_on);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "psun"));

	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.Psun);//
printf("%lf\n",gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget)));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "startvoltage"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.Vstart);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "stopvoltage"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.Vstop);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "stepvoltage"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.Vstep);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "ddclamp"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.electrical_clamp);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "eclamp"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.thermal_clamp);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "hclamp"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.thermal_clamp);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "dt"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.dt/1e-9);


	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "onvalue"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget),gui_device.onvalue);

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "offvalue"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget),gui_device.offvalue);

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "pulsestart"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget),gui_device.pulsestart);

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "reprate"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget),gui_device.reprate);

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "pulselength"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget),gui_device.pulselen);


	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "electricalerror"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), fabs(log10(gui_device.min_cur_error)));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "tleft"));

	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), 300);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "tr"));
	gtk_spin_button_set_value (GTK_SPIN_BUTTON ( widget), gui_device.Tlr);


}


void on_configure_ok (GtkObject *object, gpointer user_data)
{
GtkWidget * widget;
	gtk_widget_show(gui.configure);
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "ex1"));
	gui_device.enable_ex = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "plotresults"));
	gui_device.enable_plot = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "eb1"));
	gui_device.nnewton = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "hb1"));
	gui_device.hnewton = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "pb1"));
	gui_device.lnewton = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "timedomain"));
	gui_device.time_on = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "voltageorlight"));
	gui_device.pulsevoltage = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "psun"));
	gui_device.Psun=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "startvoltage"));
	gui_device.Vstart=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "stopvoltage"));
	gui_device.Vstop=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "stepvoltage"));
	gui_device.Vstep=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "ddclamp"));
	gui_device.electrical_clamp=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "eclamp"));
	gui_device.thermal_clamp=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget) );
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "hclamp"));
	gui_device.thermal_clamp=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));
	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "dt"));
	gui_device.dt=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget))*1e-9;

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "onvalue"));
	gui_device.onvalue=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "offvalue"));
	gui_device.offvalue=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "pulsestart"));
	gui_device.pulsestart=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "reprate"));
	gui_device.reprate=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "pulselength"));
	gui_device.pulselen=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));



	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "electricalerror"));
	gui_device.min_cur_error=pow(10.0,-gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget)));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "tleft"));
	gui_device.Tll=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

	widget = GTK_WIDGET (gtk_builder_get_object (gui.builder, "tr"));
	gui_device.Tlr=gtk_spin_button_get_value (GTK_SPIN_BUTTON ( widget));

save_config(gui.filename,&gui_device);
gtk_widget_hide(gui.configure);
}


void configure_hide (GtkObject *object, gpointer user_data)
{

	gtk_widget_hide(gui.configure);
}

void meshedit_show (GtkObject *object, gpointer user_data)
{
GtkWidget       *window=NULL;
do_mesh_edit (&gui_device,window);
}


void show_epitaxy_builder (GtkObject *object, gpointer user_data)
{
GtkWidget       *window=NULL;
do_editable_cells (&gui_device,window);
}


void show_iv (GtkObject *object, gpointer user_data)
{
	FILE *plot = popen("gnuplot -persist","w");
	fprintf(plot, "set terminal x11 title 'Solarsim - JV curve'\n");
	fprintf(plot, "clear\n");
	fprintf(plot, "set data style lines\n");
	fprintf(plot, "load 'iv.plot'\n");
	fflush(plot);
	fclose(plot);
}

void stopsimulation (GtkObject *object, gpointer user_data)
{
printdebug( "Halting simulation\n");
	cell.stop=TRUE;
}


void show_converge (GtkObject *object, gpointer user_data)
{
	FILE *plot = popen("gnuplot -persist","w");
	fprintf(plot, "set terminal x11 title 'Solarsim - Convergence'\n");
	fprintf(plot, "clear\n");
	fprintf(plot, "set data style lines\n");
	fprintf(plot, "load 'converge.plot'\n");
	fflush(plot);
	fclose(plot);
}

void showthermal (GtkObject *object, gpointer user_data)
{
	FILE *plot = popen("gnuplot -persist","w");
	fprintf(plot, "set terminal x11 title 'Solarsim - Thermal profile'\n");
	fprintf(plot, "clear\n");
	fprintf(plot, "set data style lines\n");
	fprintf(plot, "load 'thermal.plot'\n");
	fflush(plot);
	fclose(plot);
}

void showpulsetrainv (GtkObject *object, gpointer user_data)
{
	FILE *plot = popen("gnuplot -persist","w");
	fprintf(plot, "set terminal x11 title 'Solarsim - Time domain pulsing voltage'\n");
	fprintf(plot, "clear\n");
	fprintf(plot, "set data style lines\n");
	fprintf(plot, "load 'pulsetrainv.plot'\n");
	fflush(plot);
	fclose(plot);
}

void showpulsetrainl (GtkObject *object, gpointer user_data)
{
	FILE *plot = popen("gnuplot -persist","w");
	fprintf(plot, "set terminal x11 title 'Solarsim - Time domain pulsing light'\n");
	fprintf(plot, "clear\n");
	fprintf(plot, "set data style lines\n");
	fprintf(plot, "load 'pulsetrainl.plot'\n");
	fflush(plot);
	fclose(plot);
}

void open_sim_file (GtkObject *object, gpointer user_data)
{
GtkWidget *dialog;

dialog = gtk_file_chooser_dialog_new ("Open Simulation File",NULL,	GTK_FILE_CHOOSER_ACTION_OPEN,GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,NULL);

if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
{
	char *filename;
	gtk_widget_hide(dialog);
	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
	strcpy(gui.filename,filename);
	load_config(filename,&gui_device,TRUE);
}else
{
gtk_widget_destroy (dialog);
}
}

gboolean splash_hide_now(GtkObject *object, gpointer data)
{
    GtkWidget * window_10_sec = (GtkWidget *) data;
    gtk_widget_hide(window_10_sec);
    return(FALSE);
}

gboolean splash_hide(gpointer data){
    GtkWidget * window_10_sec = (GtkWidget *) data;
    gtk_widget_hide(window_10_sec);
    return(FALSE);
}


#endif

void solve_all(struct device *in)
{
solve_cur(in);
}

void dialog_set_progress (double percent)
{
#ifdef makegui
if (gui.show==TRUE)
{
		printf ("percent=%e\n", percent);
		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(gui.progress), percent);
;

	quick_update();
}
#endif
}

void device_init(struct device *in)
{
	in->Voc=0.0;
	in->Jsc=0.0;
	in->FF=0.0;
	in->Pmax=0.0;
	in->Pmax_voltage=0.0;
	device_get_memory(in);

}

void find_n0(struct device *in)
{
dprintf("Finding n0\n");
double oldsun=in->Psun;
double oldv=in->Vapplied;
in->Vapplied=0;
in->Psun=0;
light_field(in,in->spectra,0.0);
dump_for_plot(in);

solve_all(in);

reset_npinit(in);
/*for (i=0;i<in->ymeshpoints;i++)
{
	in->nfinit[i]=in->n[i];
	in->pfinit[i]=in->p[i];
	in->ntinit[i]=in->nt_all[i];
	in->ptinit[i]=in->pt_all[i];
}*/

in->Psun=oldsun;
in->Vapplied=oldv;
light_field(in,in->spectra,0.0);

}

void quick_update()
{
#ifdef makegui
if (gui.show==TRUE)
{
	int i=0;
	while ((gtk_events_pending ())&&(i<100))
	{

		gtk_main_iteration ();
		i++;

	}
}
#endif
}

int run_simulation(char *runpath)
{
if (strcmp(runpath,"")!=0) strcpy(cell.outputpath,runpath);
int i;
printf("SolarSim V0.1 License: This software is not open source and the source code or the executable may not be distributed.   I do not mind you using this code for your academic research, however, due to the large amount of time I have invested in the software I must insist that I am included as an author on any resulting publications.  rod@roderickmackenzie.eu\n");

cell.kTq=(300.0*kb/q);
//system("mv rbi.dat rbi.old");
//system("mv jv.dat jv.old");
//system("mv charge.dat charge.old");
//system("mv ivexternal.dat ivexternal.old");
printf("load config\n");
load_config("blom_device.inp",&cell,FALSE);
printf("dos init\n");
dos_init();
//load_dos(&cell,"./dosn_mb.dat","./dosp_mb.dat",1);
printf("load dos\n");
load_dos(&cell,"./dosn_gau.dat","./dosp_gau.dat");
device_init(&cell);

FILE *file;
if ((file=fopen("./rsyncdir.inp","r"))==NULL)
{
	printf("Rsync path not found\n");
	exit(0);
}

char path[1000];
fscanf(file,"%s",path);
fclose(file);
sprintf(cell.rsynccommand,"cp ./*.dat ./*.inp %s\n",path);


#ifdef dump_converge
cell.converge = fopena(cell.outputpath,"./converge.dat","w");
fclose(cell.converge);
#endif
cell.tconverge=fopena(cell.outputpath,"./tconverge.dat","w");
fclose(cell.tconverge);


int curlayer=0;
double end=cell.mat.l[0].height;

for (i=0;i<cell.ymeshpoints;i++)
{
	if (cell.ymesh[i]>=end)
	{
		curlayer++;
		end+=cell.mat.l[curlayer].height;
	}
	cell.ymatlayers[i]=curlayer;
	cell.Nad[i]=cell.mat.l[cell.ymatlayers[i]].Nad;

}
//getchar();

	for (i=0;i<cell.ymeshpoints;i++)
	{
		cell.Tl[i]=cell.Tll+cell.ymesh[i]*(cell.Tlr-cell.Tll)/cell.ylen;
		cell.Te[i]=cell.Tll+cell.ymesh[i]*(cell.Tlr-cell.Tll)/cell.ylen;
		cell.Th[i]=cell.Tll+cell.ymesh[i]*(cell.Tlr-cell.Tll)/cell.ylen;
		cell.ex[i]=0.0;
		cell.Hex[i]=0.0;
		//if ((i>cell.ymeshpoints/2)&&(i<cell.ymeshpoints/2+10)) cell.Hex[i]=1e9;
		cell.epsilonr[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_epsilonr);
		cell.dostype[i]=(int)get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_dostype);

		cell.Eg[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_Eg);//+fabs(dos_get_Ev_edge(cell.dostype[i]))+fabs(dos_get_Ec_edge(cell.dostype[i]));
		//printf("Eg=%e\n",cell.Eg[i]);
		cell.Dex[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_Dex);

		cell.Xi[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_Xi);//-fabs(dos_get_Ec_edge(cell.dostype[i]));
		//printf("Xi=%e\n",cell.Xi[i]);
		//getchar();
//printf("edge---------------|%e %e\n",dos_get_Ev_edge(cell.dostype[i]),dos_get_Ec_edge(cell.dostype[i]));
//getchar();
		cell.Ec[i]=-cell.Xi[i];

		cell.Ev[i]=-cell.Xi[i]-cell.Eg[i];

		//printf("%d %e %e\n",i,cell.mun[i],cell.mup[i]);

		cell.me[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_me);
		cell.mh[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_mh);
		cell.Nc[i]=2.0*pow(((2.0*pi*kb*cell.Tl[i]*cell.me[i]*m0)/(hp*hp)),1.5);

		cell.Nv[i]=2.0*pow(((2.0*pi*kb*cell.Tl[i]*cell.mh[i]*m0)/(hp*hp)),1.5);

		cell.mun[i]=get_n_mu();
		cell.mup[i]=get_p_mu();

		cell.kf[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_kf);
		cell.a[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_a);
		cell.kl[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_kl);
		cell.ke[i]=get_n_mu();
		cell.kh[i]=get_p_mu();

		cell.Hl[i]=0.0;
		cell.He[i]=0.0;
		cell.Hh[i]=0.0;
		cell.Hasorb[i]=0.0;

		//Electron capture
		//getchar();


		//printf("%e %e %e %e\n",cell.taue0,cell.taue1,cell.tauh0,cell.tauh1);
		//getchar();
		cell.Nct[i]=2.0*pow(((2.0*pi*kb*cell.Tl[i]*cell.me[i]*m0)/(hp*hp)),1.5);	//less dense than bulk states

		cell.Nvt[i]=2.0*pow(((2.0*pi*kb*cell.Tl[i]*cell.mh[i]*m0)/(hp*hp)),1.5);
		cell.A[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_A);
		cell.deltaE[i]=get_mat_param(&(cell.mat.l[cell.ymatlayers[i]]),mat_deltaE);
		cell.capeff[i]=get_mat_param_poly(&(cell.mat.l[cell.ymatlayers[i]]),mat_capeff,0.0);

	}
//getchar();

	for (i=0;i<cell.ymeshpoints;i++)
	{
		cell.phi[i]=0.0;
		cell.R[i]=0.0;
		cell.n[i]=0.0;
	}


light_field(&cell,cell.spectra,0.0);



update_arrays(&cell);
time_init(&cell);

cell.Vapplied=0.0;

get_initial(&cell);



solve_pos(&cell);
solver_get_memory(&cell);

if (cell.onlypos==TRUE)
{
	device_free(&cell);
	return 0;
}

plot_open(&cell);

cell.stop=FALSE;
cell.go_time=FALSE;




plot_now(&cell,"plot");



FILE *iv;
if (cell.time==FALSE)
{
iv=fopena(cell.outputpath,"./jv.dat","w");
fclose(iv);
iv=fopena(cell.outputpath,"./charge.dat","w");
fclose(iv);
iv=fopena(cell.outputpath,"./ivexternal.dat","w");
fclose(iv);
iv=fopena(cell.outputpath,"./avgmu.dat","w");
fclose(iv);
iv=fopena(cell.outputpath,"./freetrapped.dat","w");
fclose(iv);
}

tau_test(&cell);

find_n0(&cell);


dirty_fit(&cell);

draw_gaus(&cell);


if (cell.simmode==sim_mode_jv)
{
	sim_jv(&cell);
}else
if (cell.simmode==sim_mode_tpc)
{
	sim_tpc(&cell);
}else
if (cell.simmode==sim_mode_ce)
{
	sim_ce(&cell);
}
else
if (cell.simmode==sim_mode_li)
{
	sim_li(&cell);
}else
if (cell.simmode==sim_mode_tpv)
{
	sim_tpv(&cell);
}else
if (cell.simmode==sim_mode_findvoc)
{
set_light_for_voc(&cell,cell.tpvvoc);
}else
if (cell.simmode==sim_mode_jvone)
{
	sim_jv_one_point(&cell);
}


#ifdef makegui
dialog_set_progress (0.0);
#endif
device_free(&cell);

plot_close(&cell);

dos_free();
solver_free_memory(&cell);

return 0;
}

struct device gui_device;



int main (int argc, char *argv[])
{
globalserver.on=FALSE;
cell.fast=FALSE;
//wire();
//printf("%le\n",read_value(0,2));
//getchar();
//copy_file("",2,1e-2);
//exit(0);
//solve_bi();
#ifdef makegui
gui.show=FALSE;

if (scanarg( argv,argc,"--gui")==TRUE)
{
	strcpy(gui.filename,"./blom_device.inp");
	load_config(gui.filename,&gui_device,TRUE);

	gtk_init (&argc, &argv);
	gui.show=TRUE;
	GdkPixbuf *window_icon;

	window_icon=gdk_pixbuf_new_from_file( "sun_icon.png", NULL);

	gtk_window_set_default_icon (window_icon);

	gui.builder = gtk_builder_new ();
	gtk_builder_add_from_file (gui.builder, "main.glade", NULL);
	gui.window = GTK_WIDGET (gtk_builder_get_object (gui.builder, "window1"));
	gtk_window_set_title(GTK_WINDOW(gui.window), "Solarsim");
	gui.progress = GTK_WIDGET (gtk_builder_get_object (gui.builder, "mainprogres"));
	gui.configure = GTK_WIDGET (gtk_builder_get_object (gui.builder, "configure"));
	gtk_window_set_title(GTK_WINDOW(gui.configure), "Solarsim - configure");
	gui.textview = GTK_WIDGET (gtk_builder_get_object (gui.builder, "textview"));
	printdebug( "Solarsim V 0.1\n");

	gtk_builder_connect_signals (gui.builder, NULL);

	gui.splash = GTK_WIDGET (gtk_builder_get_object (gui.builder, "splash"));
	g_timeout_add (2000, splash_hide, gui.splash);

	gtk_widget_show (gui.window);
	gtk_widget_show (gui.splash);


	gtk_main ();
}else
#endif

double a;
double b;
double c;
double d;

int pos=0;
if ((pos=get_arg_plusone_pos( argv,argc,"--outputpath")!=FALSE))
{
	strcpy(cell.outputpath,argv[pos+1]);
}else
{
	strcpy(cell.outputpath,"./");
}

if (scanarg( argv,argc,"--tab")==TRUE)
{
dump_talbe();
exit(0);
}else
if (scanarg( argv,argc,"--processce")==TRUE)
{
process_ce_data("/home/rod/hopv/downsampled/ce0.dat","./ce0.dat");
process_ce_data("/home/rod/hopv/downsampled/ce100.dat","./ce100.dat");
exit(0);
}else
if (scanarg( argv,argc,"--qe")==TRUE)
{
dump_qe();
exit(0);
}else
if (scanarg( argv,argc,"--fittpv")==TRUE)
{
double start;
double stop;
sscanf(argv[3],"%le",&start);
sscanf(argv[4],"%le",&stop);
sscanf(argv[5],"%le",&a);
sscanf(argv[6],"%le",&b);
sscanf(argv[7],"%le",&c);


fit_tpv(argv[2],start,stop,a,b,c);
exit(0);
}else
if (scanarg( argv,argc,"--fittpc")==TRUE)
{
//8.00834e-08,1.66422e-07,9.02609e-07,4.34910e-06

sscanf(argv[3],"%le",&a);
sscanf(argv[4],"%le",&b);
sscanf(argv[5],"%le",&c);
sscanf(argv[6],"%le",&d);

get_tpc_t0(argv[2],a,b,c,d);
exit(0);
}else
if (scanarg( argv,argc,"--jointpvandce")==TRUE)
{
load_config("blom_device.inp",&cell,FALSE);
join_tpv_to_ce(&cell);
exit(0);
}else
if (scanarg( argv,argc,"--doubledos")==TRUE)
{
dos_double_res();
exit(0);
}

server_init(&globalserver);

if (scanarg( argv,argc,"--1fit")==TRUE)
{
static struct fitvars fitconfig;
fit_read_config(&fitconfig);
fit_run_sims(&fitconfig,&globalserver);

}else
if (scanarg( argv,argc,"--fit")==TRUE)
{

FILE *out;
out = fopen("./fitlog.dat","w");
fclose(out);

int docopy=TRUE;
FILE *file;
if ((file=fopen("./rsyncdir.inp","r"))==NULL)
{
	docopy=FALSE;
}



char path[1000];
char command[1000];

if (docopy==TRUE)
{
fscanf(file,"%s",path);
fclose(file);
sprintf(command,"cp ./*.dat ./*.inp %s\n",path);
}
	cell.fast=TRUE;
	do
	{
	fit_now();

	if (docopy==TRUE)
	{
		printf("Running command %s\n",command);
		system(command);
	}
	//getchar();
	}while(1);



}else
{
	gen_dos_fd_gaus_fd();

	server_add_job(&globalserver,"./");
	print_jobs(&globalserver);
	server_run_jobs(&globalserver);
}
server_shut_down(&globalserver);

return 0;
}

