// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>
#include <gsl/gsl_multimin.h>


static struct istruct tpcexp;

static double error_start;
static double error_stop;

double f0(double a,double b,double t)
{
return fabs(b)*pow(t,a);
}


double get_tpc_error0(double a,double b)
{
double start=0.0;
double stop=0.0;
start=error_start;
stop=error_stop;
double error=0.0;
double pos=0.0;
double points=0.0;
double etemp;
int i=0;




for (i=0;i<tpcexp.len;i++)
{
pos=tpcexp.x[i];

	if ((pos>=start)&&(pos<=stop))
	{
		etemp=fabs(log10(tpcexp.data[i])-log10(f0(a,b,pos)));
		error+=etemp;
		points+=1.0;
	}
}

return error;
}


double my_tpc_f0 (const gsl_vector *v, void *params)
{
double e0;
int item=0;
double a,b;
a = gsl_vector_get(v, item);
item++;
b = gsl_vector_get(v, item);
item++;

e0=get_tpc_error0(a,b);

return e0;
}


double fit_tpc(char * file,double start,double stop,double *aa,double *bb,int flag)
{

//inter_load_a(&tpcexp,file,0,1);

//inter_log_y(&tpcexp);
//inter_log_x(&tpcexp);

//inter_save(&tpcexp,"./a.dat");
//inter_free(&tpcexp);

char exp_name[20];
sprintf(exp_name,file);
inter_load_a(&tpcexp,exp_name,0,1);
FILE *out;
out = fopen("./fitlog.dat","w");
fclose(out);
error_start=start;
error_stop=stop;
double val=0.0;
int fitvars=2;
size_t iter = 0;
int status;
double size;
double shift=0.1;
double par[5] = {1.0, 2.0, 10.0, 20.0, 30.0};
const gsl_multimin_fminimizer_type *T = gsl_multimin_fminimizer_nmsimplex;
double a=0.0;
double b=0.0;
gsl_multimin_fminimizer *s = NULL;
gsl_vector *ss, *x;
gsl_multimin_function minex_func;



x = gsl_vector_alloc (fitvars);

ss = gsl_vector_alloc (fitvars);

int item=0;
int i;

val=-0.1;
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
val=-1.669462e-9;
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;


       minex_func.n = item;
       minex_func.f = my_tpc_f0;
       minex_func.params = par;

       s = gsl_multimin_fminimizer_alloc (T, item);
       gsl_multimin_fminimizer_set (s, &minex_func, x, ss);
     int loop=0;

do
{
	iter++;
	status = gsl_multimin_fminimizer_iterate(s);

	if (status) break;

	size = gsl_multimin_fminimizer_size (s);
	status = gsl_multimin_test_size (size, 1e-10);

	/*if (status == GSL_SUCCESS)
	{
		printf ("converged to minimum at\n");
	}*/

	a=gsl_vector_get(s->x, 0);
	b=gsl_vector_get(s->x, 1);

	//printf ("%d %le %le %le\n", (int)iter, s->fval, a,b);
	//getchar();


	loop++;

	if (loop>400)
	{
		break;
	}


}while (status == GSL_CONTINUE);
FILE *tpvfit=fopen("tpcfit.dat","w");
if (flag==0)
{
tpvfit=fopen("tpcfit0.dat","w");
}else
{
tpvfit=fopen("tpcfit1.dat","w");
}
for (i=0;i<tpcexp.len;i++)
{
	fprintf(tpvfit,"%e %e %e\n",tpcexp.x[i],tpcexp.data[i],f0(a,b,tpcexp.x[i]));
}
fclose(tpvfit);

inter_free(&tpcexp);
gsl_vector_free(x);
gsl_vector_free(ss);
gsl_multimin_fminimizer_free (s);
*aa=a;
*bb=fabs(b);

//FILE *tauout=fopen("tau.dat","w");
//fprintf(tauout,"%e %e\n",read_value("blom_device.inp",0,53),c);
//fclose(tauout);
return a;
}


double get_tpc_t0(char * file,double start0,double stop0,double start1,double stop1)
{
double a0;
double b0;
double a1;
double b1;
double alpha0;
double alpha1;
fit_tpc(file,start0,stop0,&a0,&b0,0);
alpha0=1.0+a0;
fit_tpc(file,start1,stop1,&a1,&b1,1);
alpha1=-a1-1.0;
//printf("%le %le\n",a0,b0);
//printf("%le %le\n",a1,b1);
//printf("alpha0=%le alpha1=%le\n",alpha0,alpha1);
printf("%le %le %le\n",pow(b1/b0,1.0/(alpha0+alpha1)),alpha0,alpha1);
return 0;
}
