// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>
#include <gsl/gsl_multimin.h>

static struct istruct experiment;

double myf(double x,double a,double b,double c)
{
return a*exp(b-x/c);
}

static double error_start;
static double error_stop;
double get_exp_error(double a,double b, double c)
{

double error=0.0;

double start=error_start;
double stop=error_stop;
double pos=0.0;
double etemp;
int i=0;

for (i=0;i<experiment.len;i++)
{
pos=experiment.x[i];

	if ((pos>=start)&&(pos<=stop))
	{
		etemp=fabs(experiment.data[i]-myf(pos,a,b,c));
		error+=etemp;
	}
}

return error;
}


double my_exp_f (const gsl_vector *v, void *params)
{
double e0;
int item=0;
double a,b,c;
a = gsl_vector_get(v, item);
item++;
b = gsl_vector_get(v, item);
item++;
c = gsl_vector_get(v, item);
item++;

e0=get_exp_error(a,b,c);

return e0;
}


void fit_tpv(char *file, double start, double stop, double a,double b, double c)
{
error_start=start;
error_stop=stop;
char exp_name[20];
sprintf(exp_name,file);
inter_load_a(&experiment,exp_name,0,1);
FILE *out;
out = fopen("./fitlog.dat","w");
fclose(out);

double val=0.0;
int fitvars=3;
size_t iter = 0;
int status;
double size;
double shift=0.1;
double pos=0.0;
double etemp=0.0;
double par[5] = {1.0, 2.0, 10.0, 20.0, 30.0};
const gsl_multimin_fminimizer_type *T = gsl_multimin_fminimizer_nmsimplex;//gsl_multimin_fminimizer_nmsimplex;

gsl_multimin_fminimizer *s = NULL;
gsl_vector *ss, *x;
gsl_multimin_function minex_func;



/* Starting point */
x = gsl_vector_alloc (fitvars);

ss = gsl_vector_alloc (fitvars);

int item=0;
int i;

val=a;
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
val=b;
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;
val=c;
gsl_vector_set (x, item, val);
gsl_vector_set (ss, item, fabs(val*shift));
item++;

       /* Initialize method and iterate */
       minex_func.n = item;
       minex_func.f = my_exp_f;
       minex_func.params = par;

       s = gsl_multimin_fminimizer_alloc (T, item);
       gsl_multimin_fminimizer_set (s, &minex_func, x, ss);
     int loop=0;

do
{
	iter++;
	status = gsl_multimin_fminimizer_iterate(s);

	if (status) break;

	size = gsl_multimin_fminimizer_size (s);
	status = gsl_multimin_test_size (size, 1e-8);

	//if (status == GSL_SUCCESS)
	//{
	//	printf ("converged to minimum at\n");
	//}

	a=gsl_vector_get(s->x, 0);
	b=gsl_vector_get(s->x, 1);
	c=gsl_vector_get(s->x, 2);

	//printf ("%d %le %le %le %le\n", (int)iter, s->fval, a,b,c);
	//getchar();


	loop++;

	if (loop>4000)
	{
		break;
	}

	/*if ((loop%30)==0)
	{
		FILE *tpvfit=fopen("tpvfit.dat","w");
		for (i=0;i<experiment.len;i++)
		{
		fprintf(tpvfit,"%e %e %e\n",experiment.x[i],experiment.data[i],a*exp((c*experiment.x[i])));
		}
		fclose(tpvfit);
	} //status == GSL_CONTINUE*/
}while (1);

FILE *tpvfiterror=fopen("./tpvfiterror.dat","w");


for (i=0;i<experiment.len;i++)
{
pos=experiment.x[i];

	if ((pos>=start)&&(pos<=stop))
	{
		etemp=fabs(experiment.data[i]-myf(pos,a,b,c));
		fprintf(tpvfiterror,"%le %le %le %le\n",pos,etemp,experiment.data[i],myf(pos,a,b,c));
	}
}
fclose(tpvfiterror);

inter_free(&experiment);
gsl_vector_free(x);
gsl_vector_free(ss);
gsl_multimin_fminimizer_free (s);


//FILE *tauout=fopen("tau.dat","w");
printf("%e\n",c);
//tauout,
//fclose(tauout);

}

